-- $Rev: 61 $

local addonName, L = ...;

ByeHakmud = {}
local BH = ByeHakmud

BH.CurrentPassengers = {}

--- Bye Hakmud Engine

function BH_SetListen4Event(enabled, eventName)
    if (enabled) then
	if (not BH_GetConfigPanel():IsEventRegistered(eventName)) then
	    BH_GetConfigPanel():RegisterEvent(eventName); 
	end
    else 
	if (BH_GetConfigPanel():IsEventRegistered(eventName)) then
	    BH_GetConfigPanel():UnregisterEvent(eventName); 
	end
    end
end

function BH_SetListen4Passenger(enabled)
    BH_SetListen4Event(enabled, "VEHICLE_PASSENGERS_CHANGED")
    BH_SetListen4Event(enabled, "PLAYER_GAINS_VEHICLE_DATA")
end

function BH_EnableByeHakmud(enabled)
    local oldEnabled = BH.Vars:IsByeHakmudEnabled();
    if (oldEnabled ~= enabled) then
	BH.Vars:SetByeHakmudEnabled(enabled);
	BH_SetListen4Passenger(BH.Vars:IsByeHakmudEnabled());
	if (BH.Vars:IsByeHakmudEnabled()) then
		BH_PostMessage (L["ByeHakmud turned on: Vendor can be ejected when the mount is invoked"]);
	else
		BH_PostMessage (L["ByeHakmud turned off: Vendor stays on the mount"]);
	end
    end
end

function BH_AddTargetAsPermission(permission)
    local targetName = GetUnitName("target", false)
    if targetName then
	BH_AddPlayerPermission(targetName, permission)
    end
    return targetName
end

function BH_IsPartyAllowsEject()
	local retVal = true;
	if BH.Vars:IsOnlyEjectWhenInParty() then
	    retVal = UnitInParty("player")
	end
	return retVal;
end

function BH_GetSeatsToEject4Mount(creatureID)
    local retVal = nil;
    
    if BH.Vars:GetMountIDToSeat() then
	retVal = BH.Vars:GetMountIDToSeat()[creatureID]
    end
    
    if retVal == nil then
	retVal = BH_MountIDToSeatDefaultList[creatureID]
    end
    
    return retVal;
end

function BH_SetSeatsToEject4Mount(creatureID, seatsToEject)
    if BH.Vars:GetMountIDToSeat() then 
	BH.Vars:GetMountIDToSeat()[creatureID] = seatsToEject
    end
end

local function BH_IsFriend(occupantName)
    local retVal = false
    for friendIdx = 1, GetNumFriends() do
	local friendName = GetFriendInfo(friendIdx)
	if friendName == occupantName then
	    retVal = true
	    break
	end
    end
    return retVal
end

local function BH_CreateNotifyMessage(occupantName, creatureSpellID)
    local retVal = string.gsub(
	BH.Vars:GetNotifyMsg(),
	"%[passenger%]",
	tostring(occupantName)
    )
    
    local link, tradelink = GetSpellLink(creatureSpellID)
    retVal = string.gsub(
	retVal,
	"%[mount%]",
	tostring(link)
    )
    return retVal
end

local BH_DefaultSeatsToEject = {}
function BH_EjectHakmud()
    local creatureID, creatureName, creatureSpellID, icon, issummoned
    function BH_GetSeatsToEject()
	function BH_CreateDefaultSeatsToEject()
	    for seatIdx = 1, UnitVehicleSeatCount("player") do
		BH_DefaultSeatsToEject[seatIdx] = seatIdx
		BH_DefaultSeatsToEject[seatIdx + 1] = nil
	    end	
	    return BH_DefaultSeatsToEject
	end 
	
	local retVal = nil;
	local companionCount = GetNumCompanions("MOUNT");
	for i = 1, companionCount do
	    creatureID, 
	    creatureName, 
	    creatureSpellID, 
	    icon, 
	    issummoned = GetCompanionInfo("MOUNT", i);
	    
	    if issummoned then
		retVal = BH_GetSeatsToEject4Mount(creatureID);
		
		if retVal == nil then
		    retVal = BH_CreateDefaultSeatsToEject()
		end
		
		break;
	    end
	end
	
	-- druid stag travel form has no creatureID but a seat
	local class, classFileName = UnitClass("player")
	if classFileName == "DRUID" and GetShapeshiftFormID() == 3 then 
	    retVal = BH_CreateDefaultSeatsToEject()
	    creatureSpellID = 783
	end
	
	return retVal;
    end

    local keepHakmud = IsShiftKeyDown();
    if (not BH.Vars:IsHakmudKeptOnShift()) then
	keepHakmud = not keepHakmud;
    end

    if ((not keepHakmud)
	and BH_IsPartyAllowsEject())
    then 
	local seatsToEject = BH_GetSeatsToEject();
	if seatsToEject then
	    for i, seat in ipairs(seatsToEject) do
		if CanEjectPassengerFromSeat(seat) then
		    local controlType, occupantName, occupantRealm, canEject, canSwitchSeats = UnitVehicleSeatInfo("player", seat)
		    if BH_IsEjectable(occupantName) then
			EjectPassengerFromSeat(seat)
			BH_PostMessage(BH_CreateNotifyMessage(occupantName, creatureSpellID), BH.Vars:GetChannelToNotify())
		    end
		end
	    end
	end
    end
end

--- Player Permissions
local BH_DefaultPlayerPermissions = {
    "target",
    "focus",
    "friend",
    "guild",
    "group"
}

local BH_LastPrePlayerRule = "focus"

local BH_PermissionRules = {
    target = function(occupantName)
	return GetUnitName("target", false) == occupantName
    end,
    focus = function (occupantName)
	return GetUnitName("focus", false) == occupantName 
    end,
    friend = function (occupantName)
	return BH_IsFriend(occupantName)
    end,
    guild = function (occupantName)
	return UnitIsInMyGuild(occupantName)
    end,
    group = function (occupantName)
	return true
    end,
}

function BH_IsEjectable(occupantName)
    local ejectable = false
    if UnitIsPlayer(occupantName) then
	for ruleName, permission in BH_PairsByKeys(BH_GetPlayerPermissions()) do
	    if permission ~= "DontCheck" then
		rule = BH_PermissionRules[ruleName]
		if (rule == nil and ruleName == occupantName) or 
		   (rule and rule(occupantName)) 
		then
		    ejectable = permission == "Deny"
		    break;
		end
	    end
	end
    else
	ejectable = true
    end
    
    return ejectable
end 

function BH_GetDefaultPlayerPermissions()
    return BH_DefaultPlayerPermissions
end

local BH_GetPlayerPermissionsRecursionBlock = false
function BH_GetPlayerPermissions()
    local retVal = BH.Vars:GetPlayerPermissions()
    
    if retVal then 
	local k,v = next(retVal)
	if k == nil and BH_GetPlayerPermissionsRecursionBlock == false then
	    BH_GetPlayerPermissionsRecursionBlock = true
	    for i, name in pairs(BH_DefaultPlayerPermissions) do
		BH_AddPlayerPermission(name, "DontCheck")
	    end
	    BH_GetPlayerPermissionsRecursionBlock = false
	end
    end
    
    return retVal
end

function BH_PairsByKeys (t)
    local a = {}
    for n in pairs(t) do 
	table.insert(a, n) 
    end
    table.sort(a, BH_PlayerPermissionsSort)
    
    local i = 0      -- iterator variable
    local iter = function ()   -- iterator function
	i = i + 1
	if a[i] == nil then 
	    return nil
	else 
	    return a[i], t[a[i]]
	end
    end
    return iter
end

function BH_PlayerPermissionsSort(a, b)
    local retVal
    
    local defaultIdxA, defaultIdxB, lastPrePlayerIdx
    for i, defaultName in ipairs(BH_DefaultPlayerPermissions) do
	if defaultName == a then defaultIdxA = i end
	if defaultName == b then defaultIdxB = i end
	if defaultName == BH_LastPrePlayerRule then lastPrePlayerIdx = i end
    end

    if defaultIdxA then
	if defaultIdxB then
	    retVal = defaultIdxA < defaultIdxB
	else
	    retVal = defaultIdxA <= lastPrePlayerIdx 
	end
    elseif defaultIdxB then
	retVal = defaultIdxB > lastPrePlayerIdx
    else 
	retVal = string.upper(a) < string.upper(b)
    end
    
    return retVal
end

function BH_GetPlayerPermission(player)
    local retVal = BH_GetPlayerPermissions()[player] 
    if retVal == nil then
	retVal = "DontCheck"
    end
    return retVal
end

function BH_AddPlayerPermission(playerName, permission)
    if playerName and BH_GetPlayerPermissions() then
	local trimmedName = playerName:gsub("^%s*(.-)%s*$", "%1")
	
	BH_GetPlayerPermissions()[trimmedName] = permission
	table.sort(BH_GetPlayerPermissions(), BH_PlayerPermissionsSort)
    end
end

function BH_DelPlayerPermission(name)
    local reallyDelete = true
    for i, defaultPermissionName in ipairs(BH_DefaultPlayerPermissions) do
	if name == defaultPermissionName then
	    reallyDelete = false
	    break
	end
    end
    
    if reallyDelete then
	BH_GetPlayerPermissions()[name] = nil
	table.sort(BH_GetPlayerPermissions(), BH_PlayerPermissionsSort)
    end
end

--- Bye Hakmud Engines Event Handlers

function BH_OnAddonLoaded(addonName)
	if (addonName == "ByeHakmud") then
		ByeHakmud_SavedVariables = ByeHakmud_SavedVariables or {}
		ByeHakmud_SavedVariablesPerCharacter = ByeHakmud_SavedVariablesPerCharacter or {}
		BH.Vars:SetSavedVariables(ByeHakmud_SavedVariables)
		BH.Vars:SetSavedVariablesPerCharacter(ByeHakmud_SavedVariablesPerCharacter)

		BH_SetListen4Passenger(BH.Vars:IsByeHakmudEnabled());
		BH_GetConfigPanel():UnregisterEvent("ADDON_LOADED");
		BH.Vars.loaded = true;
	end
end

function BH_OnPlayerGainsVehicleData()
    for i = 1, #BH.CurrentPassengers do
	BH.CurrentPassengers[i] = nil
    end
end

function BH_OnVehiclePassengersChanged()
    local changeDetected
    for seatIdx = 1, UnitVehicleSeatCount("player") do
	local 
	controlType, 
	occupantName, 
	occupantRealm, 
	canEject, 
	canSwitchSeats = UnitVehicleSeatInfo("player", seatIdx)
	
	-- mark empty seats as false instead of nil to avoid gaps in the BH.CurrentPassengers
	if occupantName == nil then 
	    occupantName = false 
	end
	
	if BH.CurrentPassengers[seatIdx] ~= occupantName then
	    changeDetected = true
	end
	
	BH.CurrentPassengers[seatIdx] = occupantName
    end
    
    if changeDetected then
	BH_EjectHakmud()
    end
end

function BH_OnEvent(self, event, arg1, ...)
    if      (event == "ADDON_LOADED") then BH_OnAddonLoaded(arg1);
    elseif (event == "PLAYER_GAINS_VEHICLE_DATA") then BH_OnPlayerGainsVehicleData();
    elseif (event == "VEHICLE_PASSENGERS_CHANGED") then BH_OnVehiclePassengersChanged();
    end
end

function BH_PostMessage(msg, channel)
    if channel== nil then channel = "local" end
    
    if channel == "local" then
	print(msg);
    else
	local chatType = nil
	
	if channel == "say" then
	    chatType = "SAY"
	elseif channel == "yell" then
	    chatType = "YELL"
	elseif UnitInParty("player") then
	    if channel == "party" then
		chatType = "PARTY"
	    elseif channel == "raid" then
		chatType = "RAID"
	    elseif channel == "battleground" then
		chatType = "BATTLEGROUND"
	    elseif channel == "auto" then
		if UnitInParty("player") then
		    chatType = "PARTY"
		    if UnitInBattleground("player") then
			chatType = "BATTEGROUND"
		    elseif UnitInRaid("player") then
			chatType = "RAID"
		    end
		end
	    end
	end
	
	if chatType then
	    SendChatMessage(msg, chatType)
	end
    end
end
