local BH = ByeHakmud 
 
 --- Config Panel View Accessors
local BH_ConfigPanel;
 
function BH_GetConfigPanel()
    return BH_ConfigPanel;
end

function BH_GetWidgetByName(name)
    local panel = BH_GetConfigPanel();
    return _G[panel:GetName()..name];
end
 
function BH_GetBHEnabledCB()
    return BH_GetWidgetByName("_CB_ByeHakmudEnabled");
end

function BH_GetBHOnlyEjectWhenInPartyCB()
    return BH_GetWidgetByName("_CB_OnlyEjectWhenInGroup");
end

function BH_GetBHChannelToNotifyCB()
    return BH_GetWidgetByName("_CB_CHANNEL_TO_NOTIFY");
end

function BH_GetBHNotifyMsgEB()
    return BH_GetWidgetByName("_EB_NOTIFY_MSG");
end

--- Config Panel View Initialization
function BH_GetConfigPanelName()
    return "Bye Hakmud"
end

function BH_AddConfigPanelToAddonOptions()
	local panel = BH_GetConfigPanel();
	panel.name = BH_GetConfigPanelName()
	panel.okay = function (self) BH_OnOk(); end;
	panel.refresh = function (self) BH_OnRefresh(); end;
	InterfaceOptions_AddCategory(panel);
end

--- Config Panel Controller

function BH_OnOk()
	local enabled = BH_GetBHEnabledCB():GetChecked() == 1;
	BH_EnableByeHakmud(enabled);

	local onlyEjectWhenInParty = BH_GetBHOnlyEjectWhenInPartyCB():GetChecked() == 1;
	BH.Vars:SetOnlyEjectWhenInParty(onlyEjectWhenInParty);

	BH.Vars:SetHakmudKeptOnShift(BH_RB_Model_IsKeptOnShift());
	
	local channelToNotify = UIDropDownMenu_GetSelectedValue(BH_GetBHChannelToNotifyCB());
	BH.Vars:SetChannelToNotify(channelToNotify)
	
	local notifyMsg = BH_GetBHNotifyMsgEB():GetText()
	BH.Vars:SetNotifyMsg(notifyMsg)
	
	BH_ApplySeatCheckBoxes()
end

function BH_OnRefresh()
	BH_GetBHEnabledCB():SetChecked(BH.Vars:IsByeHakmudEnabled());
	BH_GetBHOnlyEjectWhenInPartyCB():SetChecked(BH.Vars:IsOnlyEjectWhenInParty());
	BH_RB_Model_SetKeptOnShift(BH.Vars:IsHakmudKeptOnShift());
end

function BH_ConfigPanel_OnLoad(frame)
	BH_ConfigPanel = frame;
	BH_AddConfigPanelToAddonOptions();

	local subFrame = _G[frame:GetName().."SubText"]
	if subFrame then 
	    subFrame:SetText(TL(
		"ByeHakmud can automatically dismount a vendor when "..
		"a mount like the Traveler's Tundra Mammoth is invoked. "..
		"That can be useful if you want to keep a seat free for a "..
		"group member to join your mount and repair his armor while riding."
	      ));
	end

	subFrame = _G[frame:GetName().."VersionText"]
	if subFrame then
	    subFrame:SetText(BH.Vars:GetVersion() .."\n" .. BH.Vars:GetAddonUrl());
	end

	frame:RegisterEvent("ADDON_LOADED");
end

--- Controller of Radio Button 'Hakmud kept on <shift>'
local BH_RB_Model_KeptOnShift;

function BH_RB_Model_IsKeptOnShift()
	return BH_RB_Model_KeptOnShift;
end

function BH_RB_Model_SetKeptOnShift(keep)
	BH_RB_Model_KeptOnShift = keep;
	BH_UpdateHakmudKeptOnShiftButtons();
end

function BH_UpdateHakmudKeptOnShiftButtons()
	local keptOnShift = BH_RB_Model_IsKeptOnShift();
	_G["ByeHakmudConfigFrame_RB_SHIFT_KEEPS"]:SetChecked(keptOnShift);
	_G["ByeHakmudConfigFrame_RB_SHIFT_KICKS"]:SetChecked(not keptOnShift);
end

---
function BH_DropDownCallback(info)
    UIDropDownMenu_SetSelectedValue (info.owner, info.value)
end

local entry = {}
function BH_CB_AddEntry(owner, text, value, tooltipText, func)
    entry.owner	= owner
    entry.text	= text
    entry.value	= value
    entry.checked = false
    entry.func = func
    
    if tooltipText then
	entry.tooltipText = tostring(tooltipText)
	entry.tooltipTitle = tostring(text)
	entry.tooltipOnButton = 1
    else
	entry.tooltipText = nil
	entry.tooltipTitle = nil
	entry.tooltipOnButton = 0
    end
    
    UIDropDownMenu_AddButton(entry)
end

function BH_CB_ChannelToNotifyInitialize(self)
    BH_CB_AddEntry(self, TL("don't post"), "none", TL("Don't post anything"), BH_DropDownCallback)
    BH_CB_AddEntry(self, TL("local"), "local", TL("Show a message locally. No other players will be bothered with this message."), BH_DropDownCallback)
    BH_CB_AddEntry(self, TL("say"), "say", TL("Say the message (/s). Nearby players will receive the message wether in your group or not."), BH_DropDownCallback)
    BH_CB_AddEntry(self, TL("auto party/raid/battleground"), "auto", TL("Automatically tries to determine wether you are in a party, raid or on a battleground and posts the message to that channel."), BH_DropDownCallback)
    BH_CB_AddEntry(self, TL("party only"), "party", TL("Posts the message in the party channel (/p)"), BH_DropDownCallback)
    BH_CB_AddEntry(self, TL("raid only"), "raid", TL("Posts the message in the raid channel (/ra)"), BH_DropDownCallback)
    BH_CB_AddEntry(self, TL("battleground only"), "battleground", TL("Posts the message in the battleground channel (/bg)"), BH_DropDownCallback)
    BH_CB_AddEntry(self, TL("yell"), "yell", TL("Yell the message (/y)"), BH_DropDownCallback)
end

function BH_CB_ChannelToNotifyOnLoad(self)
    UIDropDownMenu_Initialize(self, BH_CB_ChannelToNotifyInitialize)
    local selVal = BH.Vars:GetChannelToNotify()
    UIDropDownMenu_SetSelectedValue(self, selVal)
end

--- Notify Msg Edit Box
function BH_EB_NotifyMSGOnShow(self)
    self:SetText("")
    self:SetText(BH.Vars:GetNotifyMsg())
end

--- Mount Selector
 
function BH_ApplySeatCheckBoxes()
    local mountSelector = BH_GetWidgetByName("_CB_MountSelector");
    local creatureID = UIDropDownMenu_GetSelectedValue(mountSelector);
    if creatureID then
	local seatsToEject = {}	    
	local seatsToEjectIdx = 1
	local checkBoxID = 1
	local checkBox = nul
	repeat
	    local checkBox = BH_GetWidgetByName("_CB_EjectVendor"..checkBoxID)
	    if checkBox then
		local checked = checkBox:GetChecked()
		if checked then
		    seatsToEject[seatsToEjectIdx] = checkBoxID
		    seatsToEjectIdx = seatsToEjectIdx + 1
		end
		checkBoxID = checkBoxID + 1
	    end
	until checkBox == nil
	BH_SetSeatsToEject4Mount(creatureID, seatsToEject)
    end
end

function BH_UpdateSeatCheckBoxes(creatureID)
    local seatsToEject = BH_GetSeatsToEject4Mount(creatureID);
    
    local checkBoxID = 1
    local checkBox = nul
    repeat
	local checkBox = BH_GetWidgetByName("_CB_EjectVendor"..checkBoxID)
	if checkBox then
	    local checked = false
	    for i, seatID in ipairs(seatsToEject) do
		if seatID == checkBoxID then
		    checked = true
		end
	    end
	    checkBox:SetChecked(checked)
	    checkBoxID = checkBoxID + 1
	end
    until checkBox == nil
end

function BH_CB_MountSelectorEntryCallback(info)
    BH_ApplySeatCheckBoxes()
    BH_DropDownCallback(info)
    BH_UpdateSeatCheckBoxes(info.value)
end

function BH_CB_MountSelectorInitialize(self)
    local companionCount = GetNumCompanions("MOUNT");
    for i = 1, companionCount do
	local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i);
	local seatsToEject = BH_GetSeatsToEject4Mount(creatureID);
	if seatsToEject then
	    BH_CB_AddEntry(self, creatureName, creatureID, nil, BH_CB_MountSelectorEntryCallback)
	end
    end
end

function BH_CB_MountSelectorOnShow(self)
    UIDropDownMenu_Initialize(self, BH_CB_MountSelectorInitialize)
    if BH.Vars:IsLoaded() then
	local selectedCreatureID = UIDropDownMenu_GetSelectedValue(self);
	
	local companionCount = GetNumCompanions("MOUNT");
	for i = 1, companionCount do
	    local creatureID, creatureName, creatureSpellID, icon, issummoned = GetCompanionInfo("MOUNT", i);
	    local seatsToEject = BH_GetSeatsToEject4Mount(creatureID);
	    if seatsToEject then
		if issummoned or selectedCreatureID == nil then
		    selectedCreatureID = creatureID
		end
	    end
	end
	UIDropDownMenu_SetSelectedValue(self, selectedCreatureID)
	BH_UpdateSeatCheckBoxes(selectedCreatureID)
    end
end

function BH_BlackWhiteListFrame_OnLoad(frame)
    local panelBWList = frame
    panelBWList.name = TL("Boarding permissions")
    _G[panelBWList:GetName().."Title"]:SetText(panelBWList.name)
    _G[panelBWList:GetName().."RuleDescription"]:SetText(
	TL("When a group member tries to board your mount these rules are "..
	    "evaluated from top to down. The first rule that matches that "..
	    "group member determines his boarding permission."
	)
    )
    panelBWList.parent = BH_GetConfigPanelName()
    InterfaceOptions_AddCategory(panelBWList)
end

local BH_RuleTooltipTexts = {
    ["target"] = {
	["Allow"]     = TL("Allows your fellow to join your mount when you have targeted him. If you have targeted something else / nothing this rule doesn't apply.");
	["Deny"]      = TL("Denies your fellow to join your mount when you have targeted him. If you have targeted something else / nothing this rule doesn't apply.");
	["DontCheck"] = TL("Disables this rule");
    };
    ["focus"] = {
	["Allow"]     = TL("Allows your fellow to join your mount when you have focussed him. If you have targeted something else / nothing this rule doesn't apply.");
	["Deny"]      = TL("Denies your fellow to join your mount when you have focussed him. If you have targeted something else / nothing this rule doesn't apply.");
	["DontCheck"] = TL("Disables this rule");
    };
    ["friend"] = {
	["Allow"]     = TL("Allows your fellow to join your mount if he is on your friend list. If he isn't listed as your friend this rule doesn't apply.");
	["Deny"]      = TL("Denies your fellow to join your mount if he is on your friend list. If he isn't listed as your friend this rule doesn't apply.");
	["DontCheck"] = TL("Disables this rule");
    };
    ["guild"] = {
	["Allow"]     = TL("Allows your fellow to join your mount if he is in your guild. If he isn't in your guild this rule doesn't apply.");
	["Deny"]      = TL("Denies your fellow to join your mount if he is in your guild. If he isn't in your guild this rule doesn't apply.");
	["DontCheck"] = TL("Disables this rule");
    };
    ["group"] = {
	["Allow"]     = TL("Allows your fellow to join your mount.");
	["Deny"]      = TL("Denies your fellow to join your mount.");
	["DontCheck"] = TL("Disables this rule");
    };    
    ["_default"] = {
	["Allow"]     = TL("Allows this player to join your mount.");
	["Deny"]      = TL("Denies this player to join your mount.");
	["DontCheck"] = TL("Disables this rule");
    };    
}

function BG_GetRuleTooltipText(ruleName, ruleType)
    local retVal
    types = BH_RuleTooltipTexts[ruleName]
    if types then
	retVal = types[ruleType]
    end
    
    if retVal == nil then
	retVal = BH_RuleTooltipTexts["_default"][ruleType]
    end
    return retVal
end

function BH_ScrollFrameUpdateFunction(self)
    local entryCount = 6
    local lineHeight = 18

    -- hide all lines and set their id to 0, the update function is responsible for showing and setting id's of valid lines	
    local entry = self:GetParent():GetName().."Entry"
    for i = 1, entryCount do
	local entryBtn = _G[ entry..i ]
	if entryBtn then
	    entryBtn:SetID(0)
	    entryBtn:Hide()
	    _G[ entry..i.."NameNormalText" ].name = nil
	end
    end
    
    local offset = FauxScrollFrame_GetOffset(self)
    local playerIdx = 1
    local playerPermissions = BH_GetPlayerPermissions()
    if playerPermissions then 
	for player, permission in BH_PairsByKeys(BH_GetPlayerPermissions()) do
	    local entryIdx = playerIdx - offset
	    if entryIdx > 0 and entryIdx <= entryCount then
		local entryName = entry..entryIdx
		if _G[ entryName.."NameNormalText" ] then
		    _G[ entryName.."NameNormalText" ].name = player
		    
		    local playerTL = player
		    local iconPrefix = "|TInterface\\FriendsFrame\\UI-Toast-FriendOnlineIcon:32|t"
		    for defPermIdx, defPerm in pairs(BH_GetDefaultPlayerPermissions()) do
			if player == defPerm then
			    playerTL = TL(player)
			    iconPrefix = "|TInterface\\Buttons\\LockButton-Locked-Up:32|t"
			    break
			end
		    end
		    
		    if permission == "Allow" then
			coloredName = iconPrefix.."|cFF00FF00"..playerTL.."|r"
		    elseif permission == "Deny" then
			coloredName = iconPrefix.."|cFFFF0000"..playerTL.."|r"
		    elseif permission == "DontCheck" then
			coloredName = iconPrefix.."|cFF7F7F7F"..playerTL.."|r"
		    end
		    
		    _G[ entryName.."NameNormalText" ]:SetText(coloredName)
		    
		    local cbTypes = {"Allow","Deny","DontCheck"}
		    for i, cbType in pairs(cbTypes) do
			local chbx = _G[ entryName..cbType ]
			chbx:SetChecked((permission == cbType) and 1 or nil)
			chbx.tooltipText = BG_GetRuleTooltipText(player, cbType)
		    end
		    
		    _G[ entryName.."DontCheck" ]:SetChecked((permission == "DontCheck") and 1 or nil)
		    _G[ entryName ]:SetID(playerIdx)
		    _G[ entryName ]:Show()
		end
	    end
	    
	    entryIdx = entryIdx + 1
	    playerIdx = playerIdx + 1
	end
    end
    
    local scrollFrame = _G[ self:GetName()]
    local scrollNumItems = playerIdx
    local scrollNumToDisplay = entryCount
    local scrollValueStep = lineHeight
    
    FauxScrollFrame_Update( scrollFrame, scrollNumItems, scrollNumToDisplay, scrollValueStep);
end

function BH_PermissionCheckbox_OnClick(checkbox)
    local parentName = checkbox:GetParent():GetName()
    local playerName = _G[ parentName.."NameNormalText" ].name

    local prefixLen = string.len(parentName) + string.len(tostring(checkbox:GetID()))
    local permission = string.sub(checkbox:GetName(), prefixLen)

    BH_GetPlayerPermissions()[playerName] = permission
    
    local sfName = checkbox:GetParent():GetParent():GetName() .."ScrollFrame"
    BH_ScrollFrameUpdateFunction(_G[sfName])

end
